"use strict";
/*
 * SonarTS
 * Copyright (C) 2017-2019 SonarSource SA
 * mailto:info AT sonarsource DOT com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const ts = require("typescript");
const navigation_1 = require("./navigation");
const nodes_1 = require("./nodes");
function getFunctionComplexityNodes(functionNode) {
    return getComplexityNodes(functionNode, true);
}
exports.getFunctionComplexityNodes = getFunctionComplexityNodes;
function getOverallComplexity(node) {
    return getComplexityNodes(node, false);
}
exports.getOverallComplexity = getOverallComplexity;
function getComplexityNodes(rootNode, skipFunctions) {
    const complexityNodes = [];
    const visitNode = (node) => {
        if (nodes_1.isFunctionLikeDeclaration(node)) {
            if (skipFunctions && node !== rootNode) {
                return;
            }
            else {
                complexityNodes.push(navigation_1.functionLikeMainToken(node));
            }
        }
        switch (node.kind) {
            case ts.SyntaxKind.ConditionalExpression:
                complexityNodes.push(node.questionToken);
                break;
            // for, for-in, for-of
            case ts.SyntaxKind.ForKeyword:
            case ts.SyntaxKind.DoKeyword:
            case ts.SyntaxKind.IfKeyword:
            case ts.SyntaxKind.CaseKeyword:
                complexityNodes.push(node);
                break;
            case ts.SyntaxKind.WhileStatement:
                const whileKeyword = navigation_1.findChild(node, ts.SyntaxKind.WhileKeyword);
                complexityNodes.push(whileKeyword);
                break;
            case ts.SyntaxKind.BinaryExpression:
                const binaryExpression = node;
                if (nodes_1.is(binaryExpression.operatorToken, ts.SyntaxKind.BarBarToken, ts.SyntaxKind.AmpersandAmpersandToken)) {
                    complexityNodes.push(binaryExpression.operatorToken);
                }
                break;
        }
        node.getChildren().forEach(visitNode);
    };
    visitNode(rootNode);
    return complexityNodes;
}
//# sourceMappingURL=cyclomaticComplexity.js.map